# Copyright (C) 2002-2004 Kouichirou Eto, All rights reserved.

require "chise/idstree"

module CHISE
  IDC_0 = "\342\277\260"
  IDC_1 = "\342\277\261"
  IDC_2 = "\342\277\262"
  IDC_3 = "\342\277\263"
  IDC_4 = "\342\277\264"
  IDC_5 = "\342\277\265"
  IDC_6 = "\342\277\266"
  IDC_7 = "\342\277\267"
  IDC_8 = "\342\277\270"
  IDC_9 = "\342\277\271"
  IDC_A = "\342\277\272"
  IDC_B = "\342\277\273"

  IDC_LEFT_TO_RIGHT = IDC_0
  IDC_ABOVE_TO_BELOW = IDC_1
  IDC_LEFT_TO_MIDDLE_AND_RIGHT = IDC_2
  IDC_ABOVE_TO_MIDDLE_AND_BELOW = IDC_3
  IDC_FULL_SURROUND = IDC_4
  IDC_SURROUND_FROM_ABOVE = IDC_5
  IDC_SURROUND_FROM_BELOW = IDC_6
  IDC_SURROUND_FROM_LEFT = IDC_7
  IDC_SURROUND_FROM_UPPER_LEFT = IDC_8
  IDC_SURROUND_FROM_UPPER_RIGHT = IDC_9
  IDC_SURROUND_FROM_LOWER_LEFT = IDC_A
  IDC_OVERLAID = IDC_B

  class IDS
    def initialize(ids)
      @ids = ids
      @ids.freeze
    end

    def tree() IDS_Tree.new(@ids); end

    def compose(dbname="ids")
      cd = ChiseDB.instance
      byidsdb = cd.get_by_ids_db(dbname)
      cid = byidsdb.decode(@ids)
      return "" if cid.nil? # TO CHECK: why "", not nil?
      composed = Character.get(cid).to_s
      return "" if composed.nil?
      return "" if composed.char_length == 0
      return composed if composed.char_length == 1
      composed.each_char {|ch|
	char = ch.char
	#return ch if char.has_attribute?
	return ch # TO CHECK: the first character?
      }
      return ""
    end

    def aggregate(dbname="ids")
      # In each sub part of IDS, search the corresponding char_id.
      # If you could search the corresponding char_id, substitute with it.
      tree = self.tree
      return @ids if tree.depth <= 1 # no sub_node
      tree.sub_nodes.each {|node|
	c = node.to_ids.compose(dbname)
	next if c.nil? || c == ""
	#      print "#{@ids}	#{node}	#{c}\n"
	#      p [@ids, node, c]
	n = @ids.gsub(node, c)
	return n.to_ids.aggregate(dbname)
      }
      @ids
    end
  end

  module StringIDS
    def decompose
      map_char {|ch| ch.char.decompose }
    end

    def decompose_all
      map_char {|ch| ch.char.decompose_all }
    end
  end

  module CharacterIDC
    def is_idc?
      0x2ff0 <= @char_id && @char_id <= 0x2fff
    end

    def idc_argument_number
      return 0 unless is_idc?
      return 3 if @char_id == 0x2ff2 || @char_id == 0x2ff3
      return 2
    end
  end

  module CharacterIDS
    def decompose # by glyph
      decompose_internal
    end

    def decompose_by_meaning
      decompose_internal(true)
    end

    def decompose_all
      pde = ""
      de = self.decompose # the start point.
      level = 0
      while true
	pde = de
	de = pde.decompose # decompose it again.
	break if pde == de # previous is same.
	exit if 10 < level # p ["too many recursive", self] 
	level += 1
      end
      de
    end

    private

    def decompose_internal(by_meaning=nil)
      #idss = self.ids
      #return idss if idss
      #return k if self.is_basic_kanji?
      #return ids if idss && 0 < ids.length && k != ids

      k = self.to_s
      if by_meaning
	ids = self.ids_represent
	return ids if ids && 0 < ids.length && k != ids
	ids = self.ids_element
	return ids if ids && 0 < ids.length && k != ids
	ids = self.ids_meaning
	return ids if ids && 0 < ids.length && k != ids
      end
      ids = self.ids
      return ids if ids && 0 < ids.length && k != ids
      ids = self.ids_org
      return ids if ids && 0 < ids.length && k != ids
      k

      #return k if ids.nil? || ids.length == 0 || k == ids
      #if ids.char_length == 2
      #p ["What???", k, ids, k.inspect_all]
      ##return idsx[1] #二個目だけ返すとか?
      #return k #IDSに展開する方法が無いと。
      #end
      #return k if k == ids
      #if ids.include?(k) #<C5-4C4D><C6-4A37>この二文字のBUG対策
      ##return ids.sub(k, "")
      #return k #IDSに展開する方法が無いと。
      #end
      #return ids
    end

  end
end
